c ---------------------------------------------------------------------
      subroutine slfzr(itrjd,omegz,ndmax,difm,bzero)
c    Calculate free (p-h) and (delta-h) polarization propagators
c    for fixed transferred J
C    Modified by M.Ichimura 2016.6
C---------------------------------------------------------------------
c  INPUT: through arguments
c    itrjd : = J+1
c    omegaz: Transferred intrinsic energy in MeV
c    ndmax : (# of transferred l) x (# of radius mesh points)
c    difm  : Nucleon-Delta mass difference
C  INPUTS: through commons
C    nblmx: number of occupied states 
C  LOCAL
C    FFGN1(402) : Regular solution  for complex energy 
C    FFGN2(402) : Singular solution for complex energy
c    DELM       : Nucleon-Delta mass difference
C  OUTPUT
c    bzero(n1,n2,ist1,ist2,NDP1,NDP2): Free polarization propagators 
c      n1=(r1,l1) : Represent the set of the radius r1 and trasferred l1
c      n2=(r2,l2) : Represent the set of the radius r2 and trasferred l2
c      ist1, ist2 : =1; spin-scalar mode, =2; spin vector modes
C      NDP1, NDP2 : =1; nucleon particle, =2; Delta particle
c----------------------------------------------------------------------
      include 'dimn.fh'
      include 'ndp.fh'
      include 'fund1.fh'
      include 'fund2.fh'
      include 'opta1.fh'
      include 'opta2.fh'
      include 'orbt.fh'
      include 'space.fh'
      include 'wave.fh'
      include 'bzero.fh'
C
      dimension slfzff(nsrdm,nsrdm),vertx(2,2,4),facs(2)
      complex slfzff
      COMPLEX WI0,TECM
      PARAMETER(WI0=(0.0,1.0))
      COMPLEX ENGY,VDEPTH
      REAL  RPUT(21)
      INTEGER IPUT(3)
      COMPLEX*16 FFGN1(402)
      COMPLEX*16 FFGN2(402)
      dimension nci(ndmocm)
c
C   { Initialize BZERO
      do 202 n2=1,ndmax
      do 202 n1=1,ndmax
      do 201 NDP1=NDPMIN,NDPMAX
      do 201 NDP2=NDPMIN,NDPMAX
      do 200 ist1=1,kstrmx
      do 200 ist2=1,kstrmx
        bzero(n1,n2,ist1,ist2,NDP1,NDP2)=(0.0,0.0)
  200 continue
  201 continue
  202 continue
C
C    Set Local Variavles
      itrjw=2*itrjd
C   
C    DO LOOP OF NEUTRON(ICH=1) AND PROTON(ICH=2) HOLE
      DO 900 ICH=1,2
      npmax=npairx(ICH)
C----------------------------------------------------------------------
      if(npmax.eq.0) goto 900
C----------------------------------------------------------------------
      nspmax=nspmx(ICH)
      is1w=2.*pspn(ICH)+0.01
C   { DO LOOP OF COUPLED PAIRS
      NPDMIN=1
      DO 800 NPD=NPDMIN,NPMAX
      ichnl=ipsd2(npd,ICH)
      ptz=pztz(ichnl)
      is2w=2.*pspn(ichnl)+0.01
c
c=====================================================================
c     { SET DELM, FACS, FACT: isospin factor and reduced spin matrix
c        facs(1) = <1/2||1||1/2> = sqrt(2)
c                = <3/2||1||1/2> = <1/2||1||3/2> = 0
c        facs(2) = <1/2||sigma||1/2> = sqrt(6)
c                = <3/2||S||1/2> = <1/2||S^dag||3/2> = 2
c        fact    = |<1/2,1/2|tau_{+1}|1/2,-1/2>|^2 = 2
c                = |<1/2,pm1/2|tau_{0}|1/2,pm1/2>|^2 = 1
c                = |<3/2,3/2|T_{+1}|1/2,1/2>|^2 = 1
c                = |<3/2,1/2|T_{+1}|1/2,-1/2>|^2 = 1/3
c                =  |<3/2,1/2|T_{0}|1/2,1/2>|^2 = 2/3
c----------------------------------------------------------------------
        if(ichnl.le.4) then
          NDP=1
          delm=0.0
          facs(1)=sqrt(2.)
          facs(2)=sqrt(6.)
C
          if(itrz.eq.0) then
            fact=1.
          else
            fact=2.
          end if
C
        else if(ichnl.le.8) then
          NDP=2
          delm=difm
          facs(1)=0.0
          facs(2)=2.0
C----------------------------------------------------------------------
          if(itrz.eq.0) then
            fact=0.66666667e0
          else
            if((ichnl.eq.5).or.(ichnl.eq.8)) then
              fact=1.
            else
              fact=0.33333333e0
            end if
          end if
        end if
c======================================================================
c
c   { DO LOOP OF HOLE STAES
      DO 600 NSPZ=1,NSPMAX
      ikhl=NSPZ+nspmx(1)*(ICH-1)
      li=lspd(NSPZ,ICH)+1
      l1w=2*lspd(NSPZ,ICH)
      j1w=2*fjspd(NSPZ,ICH)+0.01
c
      TECM=OMEGZ*FBCK(NPD,ICH)-BERSPD(NSPZ,ICH)
     1     +WI0*BEISPD(NSPZ,ICH)-DELM
      AECM=REAL(TECM)
C
C     { Warning: against 0-devision
       if(abs(aecm).lt.0.05) then
          write(6,1900) tecm,omegz
 1900     FORMAT(10X,'** WARNNING IN SLFZR: ENERGY SHIFTED FROM ECM ='
     1    ,F8.3,F8.3,' MEV ( OMEG =',F8.3,' MEV *')
          TECM=TECM-AECM
          if(aecm.lt.0.0) then
            aecm=-0.05
          else
            aecm= 0.05
          end if
          TECM=TECM+AECM
        end if
C
C   { Set Lmin & Lmax
      if(kltrmx.eq.1) then
        lmin=iabs(li-1-ltr(1))+1
        lmax=li+ltr(1)
      else if(kltrmx.eq.2) then
        lmin=min0(iabs(li-1-ltr(1)),iabs(li-1-ltr(2)))+1
        lmax=li+max0(ltr(1),ltr(2))
      end if
C
      mxsm=is2w+1
      ljmax=mxsm
      if(abs(vsod(ichnl)).lt.0.01) ljmax=1
      if(pspn(ichnl).gt.1.01) ljmax=1
c   {
      multin=2
c      if(ichnl.gt.4) multin=4
      nrmax=nsrmax*multin
      dr=dsr/float(multin)
C   {
      nrlog=nrmax/2
      nr3max=nrmax+3
C   {
      t1=pmas(ichnl)+tmas(ichnl)
      fmud=pmas(ichnl)*tmas(ichnl)/t1
C
C { INPUT FOR CXFH (I)
      DRX=DR
      NRAMAX=NRMAX
      ENGY=TECM
      VDEPTH=CMPLX(VD(ICHNL),WD(ICHNL))
      IF(ABS(WD(ICHNL)).GT.998) THEN
         WDEPTH=SPWD(REAL(ENGY),BEFM(ICHNL),
     &      alphsw(ICHNL),eps0sw(ICHNL),eps1sw(ICHNL))
         IF(WDEPTH.LT.0.1) WDEPTH=0.1
         VDEPTH=VD(ICHNL)+ WI0*WDEPTH
      END IF
      RPUT(1)=PMAS(ICHNL)
      RPUT(2)=TMAS(ICHNL)
      RPUT(3)=RRD(ICHNL)
      RPUT(4)=ARD(ICHNL)
      RPUT(5)=WV0(ICHNL)
      RPUT(6)=WS0(ICHNL)
      RPUT(7)=RVD2(ICHNL)
      RPUT(8)=AVD2(ICHNL)
      RPUT(9)=RID(ICHNL)
      RPUT(10)=AID(ICHNL)
      RPUT(11)=VSOD(ICHNL)
      RPUT(12)=RSOD(ICHNL)
      RPUT(13)=ASOD(ICHNL)
      RPUT(14)=PZTZ(ICHNL)
      RPUT(15)=REFFD(ICHNL)
      RPUT(16)=AEFFD(ICHNL)
      RPUT(17)=BDIFD(ICHNL)
      RPUT(21)=rcd(ichnl)
C     }
      aecm=abs(aecm)
      fkay=0.2195376e0*sqrt(fmud*aecm)
      eta=0.15805086e0*pztz(ichnl)*sqrt(fmud/aecm)
      fkdr=fkay*dr
      fkdr56=0.83333333e0*(fkdr*fkdr)
      fkdr12=fkdr56*0.1
      t2=fkay*tmas(ichnl)**0.33333333e0
      rhobn=rrd(ichnl)*t2
      rhoin=rid(ichnl)*t2
      rhosor=rsod(ichnl)*t2
      rhobc=rcd(ichnl)*t2
      fkayar=ard(ichnl)*fkay
      fkayai=aid(ichnl)*fkay
      fkasor=asod(ichnl)*fkay
      vbe=abs(vd(ichnl))/aecm
      wbe=abs(wd(ichnl))/aecm
      vsoe=vsod(ichnl)/aecm
      mltn2=multin/2
      nrmax2=2*nsrmax
C
C   { Initialize
      do 236 irom=1,13
        lmad(irom)=0
        ljmad(irom)=0
  236 continue
      do 238 lm=1,30
        do 238 ljm=1,2
          nadljm(lm,ljm)=0
  238 continue
C   }
      irom=1
c
c   { DO LOOP OF L-PARTICLE
      do 410 lm=lmin,lmax,2
C
C     { INITIALIZE
        do 239 lt=1,2
        do 239 ljm=1,4
        do 239 ist=1,2
          vertx(lt,ist,ljm)=0.0
  239   continue
c
c     { DO LOOP OF J-PARTICLE
        do 400 ljm=1,mxsm
        lc=lm-1
        fjl=ljm+lm-2
        fjc=fjl-pspn(ichnl)
        l2w=2*lc
        j2w=2.*fjc+0.01
C
C     { INPUT FOR CXFH (II)
        IPUT(1)=LC
        IPUT(2)=IS2W
        IPUT(3)=J2W
C     }
c   --- kinematical factors ---
C----------------------------------------------------------------------
        if(fjc.lt.-0.01) goto 400
C----------------------------------------------------------------------
        fac1=sqrt(float((j1w+1)*(j2w+1)*(l1w+1)*(l2w+1))/12.56637)
C     { Set VERTX
        do 247 ist=kstrmn,kstrmx
          itrsw=2*istr(ist)
        do 245 lt=1,kltrmx
          ltrw=2*ltr(lt)
          fac2=(-1.)**((ltrw-l1w-l2w)/4)*cleb(l1w,0,l2w,0,ltrw,0)
     1        *u9(l2w,is2w,l1w,is1w,j2w,j1w,ltrw,itrsw,itrjw)
          vertx(lt,ist,ljm)=fac1*fac2*facs(ist)
  245   continue
  247   continue
c
        nm=nadljm(lm,1)
        if(ljmax.ne.1) nm=nadljm(lm,ljm)
C     {
        if(nm.le.0) then
          if(irom.gt.13) irom=1
C
C------ two independent solutions -----------------------------------
      CALL CXFH(DRX,NRAMAX,IPUT,ENGY,VDEPTH,RPUT,FFGN1,FFGN2)
C----------------------------------------------------------------------
         NR=MLTN2
         DO 260 NR2=1,NRMAX2
           CMF(NR2,IROM)=FFGN1(NR)
           NR=NR+MLTN2
 260     CONTINUE
         NR=MLTN2
         DO 280 NR2=1,NRMAX2
           CMH(NR2,IROM)=FFGN2(NR)
           NR=NR+MLTN2
 280     CONTINUE
C
          lm0=lmad(irom)
          if(lm0.gt.0) then
            ljm0=ljmad(irom)
            nadljm(lm0,ljm0)=0
          end if
C
          nadljm(lm,ljm)=irom
          lmad(irom)=lm
          ljmad(irom)=ljm
          nm=irom
          irom=irom+1
        end if
C
c     ORTHOGONALIZATION OF PARTICLE GREEN FUNCTION
C
      nbl=0
      if(ichnl.le.4) then
         ICH2=ichnl-2
         do norb=1,nspmx(ICH2)
            if(lspd(norb,ICH2)-lm+1.eq.0) then
               if(abs(fjspd(norb,ICH2)-fjc).le.0.1) then
                  nbl=nbl+1
                  nci(nbl)=norb+nspmx(1)*(ICH2-1)
               end if
            end if
         end do
      end if
      nblmx=nbl
c
      kinmin=ljm
      if(ljmax.le.1) then
C----------------------------------------------------------------------
        if(ljm.lt.mxsm) goto 400
C----------------------------------------------------------------------
        kinmin=1
      end if
c
C----------------------------------------------------------------------
      CALL OCMG(nci,nblmx,ikhl,nm,slfzff,tecm,fmud)
C          input : nblmx,ikhl,nm, ichnl
C          output: slfzff
C----------------------------------------------------------------------
C
C  -- do loops of transferred l (l1 and l2)
C
      do 360 lt2=1,kltrmx
        do 350 lt1=1,kltrmx
            do 349 ist2=kstrmn,kstrmx
              do 348 ist1=kstrmn,kstrmx
          sum=0.0
          do 335 kin=kinmin,ljm
            sum=sum+vertx(lt1,ist1,kin)*vertx(lt2,ist2,kin)
  335     continue
          fac=fact*sum
C----------------------------------------------------------------------
          if(abs(fac).lt.1.0e-5) goto 348
C----------------------------------------------------------------------
          do 345 n2=1,nsrmax
          i2=n2+nsrmax*(lt2-1)
            do 340 n1=1,nsrmax
            i1=n1+nsrmax*(lt1-1)
            bzero(i1,i2,ist1,ist2,NDP,NDP)=
     1      bzero(i1,i2,ist1,ist2,NDP,NDP)+fac*slfzff(n1,n2)
  340       continue
  345     continue
  348         continue
  349       continue
  350   continue
  360 continue
  400 continue
  410 continue
  600 continue
  800 continue
  900 continue
C
      return
      end
